package es.uv.garcosda.repository;


import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import es.uv.garcosda.repository.domain.Video;
import es.uv.garcosda.repository.repos.VideoRepository;


@RestController
@RequestMapping("/api/v1/videos")
public class VideoController {
	private final static Logger LOGGER = LoggerFactory.getLogger(VideoController.class);
	
	@Autowired private VideoRepository vr;
	
	@GetMapping("")
	public ResponseEntity<List<Video>> findVideos(@RequestParam(value="title", required = false) Optional<String> title, 
			@RequestParam(value="user", required = false) Optional<String> userId) {
		
		List<Video> videos = new ArrayList<Video>();
		
		if(!title.isPresent() && !userId.isPresent()) {
			LOGGER.debug("Get all videos");
			videos = vr.findAll();
		}
		else if(title.isPresent()) {
			LOGGER.debug("Get videos by title");
			videos = vr.findByTitle(title.get());
		}
		else if(userId.isPresent()) {
			LOGGER.debug("Get videos by user");
			videos = vr.findByUserId(userId.get());
		}	
		
		return new ResponseEntity<List<Video>>(videos, HttpStatus.OK);
	}
	
	@GetMapping("{id}")
	public Video findVideoById(@PathVariable("id") String id) {
		LOGGER.debug("Get video id: "+id);
		Optional<Video> video = vr.findById(id);
		
		if(video.isEmpty()) {
			return new Video();
		}
		else {
			return video.get();
		}
	}
	
	@PostMapping()
	public ResponseEntity<Video> createVideo(@RequestBody Video video) {
		LOGGER.debug("Create video");
		Video createdVideo = vr.save(video);
		
		if(createdVideo.getId() != null) {
			return new ResponseEntity<Video>(createdVideo, HttpStatus.CREATED);
		}
		else {
			return new ResponseEntity<Video>(new Video(), HttpStatus.BAD_REQUEST);
		}
	}
			
	@DeleteMapping("{id}")
	public ResponseEntity<String> deleteVideoById(@PathVariable("id") String id) {
		LOGGER.debug("Delete video id: "+id);
		vr.deleteById(id);
		return new ResponseEntity<String>("", HttpStatus.OK);
	}
	
}
